@@ -168,6 +168,20 @@ class Config(object):
         of endpoint URLs provided via environment variables and
         the shared configuration file.
 
+        Defaults to None.
+
+    :type request_min_compression_size_bytes: int
+    :param request_min_compression_bytes: The minimum size in bytes that a
+    request body should be to trigger compression. All requests with streaming
+    input that don't contain the `requiresLength` trait will be compressed
+    regardless of this setting.
+
+        Defaults to None.
+
+    :type disable_request_compression: bool
+    :param disable_request_compression: Disables request body compression if
+    set to True.
+
         Defaults to None.
     """
     OPTION_DEFAULTS = OrderedDict([
@@ -189,6 +203,8 @@ class Config(object):
         ('use_dualstack_endpoint', None),
         ('use_fips_endpoint', None),
         ('ignore_configured_endpoint_urls', None),
+        ('request_min_compression_size_bytes', None),
+        ('disable_request_compression', None),
     ])
 
     def __init__(self, *args, **kwargs):
