@@ -12,6 +12,8 @@
 # language governing permissions and limitations under the License.
 
 import re
+
+from awscli.customizations.exceptions import ParamValidationError
 from awscli.customizations.commands import BasicCommand
 from awscli.customizations.emr import applicationutils
 from awscli.customizations.emr import argumentschema
@@ -116,6 +118,10 @@ class CreateCluster(Command):
          'help_text' : helptext.CUSTOM_AMI_ID},
         {'name': 'ebs-root-volume-size',
          'help_text' : helptext.EBS_ROOT_VOLUME_SIZE},
+        {'name': 'ebs-root-volume-iops',
+         'help_text' : helptext.EBS_ROOT_VOLUME_IOPS},
+        {'name': 'ebs-root-volume-throughput',
+         'help_text' : helptext.EBS_ROOT_VOLUME_THROUGHPUT},
         {'name': 'repo-upgrade-on-boot',
          'help_text' : helptext.REPO_UPGRADE_ON_BOOT},
         {'name': 'kerberos-attributes',
@@ -187,12 +193,14 @@ class CreateCluster(Command):
                     params["Configurations"] = json.loads(
                         parsed_args.configurations)
                 except ValueError:
-                    raise ValueError('aws: error: invalid json argument for '
-                                     'option --configurations')
+                    raise ParamValidationError(
+                        'aws: error: invalid json argument for '
+                        'option --configurations'
+                    )
 
         if (parsed_args.release_label is None and
                 parsed_args.ami_version is not None):
-            is_valid_ami_version = re.match('\d?\..*', parsed_args.ami_version)
+            is_valid_ami_version = re.match(r'\d?\..*', parsed_args.ami_version)
             if is_valid_ami_version is None:
                 raise exceptions.InvalidAmiVersionError(
                     ami_version=parsed_args.ami_version)
@@ -343,6 +351,14 @@ class CreateCluster(Command):
             emrutils.apply_dict(
                 params, 'EbsRootVolumeSize', int(parsed_args.ebs_root_volume_size)
             )
+        if parsed_args.ebs_root_volume_iops is not None:
+            emrutils.apply_dict(
+                params, 'EbsRootVolumeIops', int(parsed_args.ebs_root_volume_iops)
+            )
+        if parsed_args.ebs_root_volume_throughput is not None:
+            emrutils.apply_dict(
+                params, 'EbsRootVolumeThroughput', int(parsed_args.ebs_root_volume_throughput)
+            )
 
         if parsed_args.repo_upgrade_on_boot is not None:
             emrutils.apply_dict(
@@ -474,7 +490,7 @@ class CreateCluster(Command):
         bootstrap_actions = []
         if len(cluster_ba_list) + len(parsed_boostrap_actions) \
                 > constants.MAX_BOOTSTRAP_ACTION_NUMBER:
-            raise ValueError('aws: error: maximum number of '
+            raise ParamValidationError('aws: error: maximum number of '
                              'bootstrap actions for a cluster exceeded.')
 
         for ba in parsed_boostrap_actions:
@@ -497,7 +513,7 @@ class CreateCluster(Command):
             bootstrap_actions.append(ba_config)
 
         result = cluster_ba_list + bootstrap_actions
-        if len(result) > 0:
+        if result:
             cluster['BootstrapActions'] = result
 
         return cluster
@@ -519,9 +535,9 @@ class CreateCluster(Command):
             args=args)
 
     def _update_cluster_dict(self, cluster, key, value):
-        if key in cluster.keys():
+        if key in cluster:
             cluster[key] += value
-        elif value is not None and len(value) > 0:
+        elif value:
             cluster[key] = value
         return cluster
 
@@ -554,14 +570,14 @@ class CreateCluster(Command):
             if constants.HBASE not in specified_apps:
                 missing_apps.add(constants.HBASE.title())
 
-        if len(missing_apps) != 0:
+        if missing_apps:
             raise exceptions.MissingApplicationsError(
                 applications=missing_apps)
 
     def _get_missing_applications_for_steps(self, specified_apps, parsed_args):
         allowed_app_steps = set([constants.HIVE, constants.PIG,
                                  constants.IMPALA])
-        missing_apps = set([])
+        missing_apps = set()
         if parsed_args.steps is not None:
             for step in parsed_args.steps:
                 if len(missing_apps) == len(allowed_app_steps):
