@@ -15,7 +15,7 @@ import logging
 import sys
 
 from botocore.client import Config
-from botocore.utils import is_s3express_bucket, ensure_boolean
+from botocore.utils import is_s3express_bucket
 from dateutil.parser import parse
 from dateutil.tz import tzlocal
 
@@ -23,6 +23,9 @@ from awscli.compat import six
 from awscli.compat import queue
 from awscli.customizations.commands import BasicCommand
 from awscli.customizations.s3.comparator import Comparator
+from awscli.customizations.s3.factory import (
+    ClientFactory, TransferManagerFactory
+)
 from awscli.customizations.s3.fileinfobuilder import FileInfoBuilder
 from awscli.customizations.s3.fileformat import FileFormat
 from awscli.customizations.s3.filegenerator import FileGenerator
@@ -31,12 +34,12 @@ from awscli.customizations.s3.filters import create_filter
 from awscli.customizations.s3.s3handler import S3TransferHandlerFactory
 from awscli.customizations.s3.utils import find_bucket_key, AppendFilter, \
     find_dest_path_comp_key, human_readable_size, \
-    RequestParamsMapper, split_s3_bucket_key, block_unsupported_resources, \
-    S3PathResolver
+    RequestParamsMapper, split_s3_bucket_key, block_unsupported_resources
 from awscli.customizations.utils import uni_print
 from awscli.customizations.s3.syncstrategy.base import MissingFileSync, \
     SizeAndLastModifiedSync, NeverSync
 from awscli.customizations.s3 import transferconfig
+from awscli.customizations.exceptions import ParamValidationError
 
 
 LOGGER = logging.getLogger(__name__)
@@ -320,10 +323,9 @@ METADATA = {
     },
     'help_text': (
         "A map of metadata to store with the objects in S3. This will be "
-        "applied to every object which is part of this request. In a sync, this "
-        "means that files which haven't changed won't receive the new metadata. "
-        "When copying between two s3 locations, the metadata-directive "
-        "argument will default to 'REPLACE' unless otherwise specified."
+        "applied to every object which is part of this request. In a sync, "
+        "this means that files which haven't changed won't receive the new "
+        "metadata. "
     )
 }
 
@@ -331,22 +333,49 @@ METADATA = {
 METADATA_DIRECTIVE = {
     'name': 'metadata-directive', 'choices': ['COPY', 'REPLACE'],
     'help_text': (
-        'Specifies whether the metadata is copied from the source object '
-        'or replaced with metadata provided when copying S3 objects. '
-        'Note that if the object is copied over in parts, the source '
-        'object\'s metadata will not be copied over, no matter the value for '
-        '``--metadata-directive``, and instead the desired metadata values '
-        'must be specified as parameters on the command line. '
-        'Valid values are ``COPY`` and ``REPLACE``. If this parameter is not '
-        'specified, ``COPY`` will be used by default. If ``REPLACE`` is used, '
-        'the copied object will only have the metadata values that were'
-        ' specified by the CLI command. Note that if you are '
-        'using any of the following parameters: ``--content-type``, '
-        '``content-language``, ``--content-encoding``, '
-        '``--content-disposition``, ``--cache-control``, or ``--expires``, you '
-        'will need to specify ``--metadata-directive REPLACE`` for '
-        'non-multipart copies if you want the copied objects to have the '
-        'specified metadata values.')
+        'Sets the ``x-amz-metadata-directive`` header for CopyObject '
+        'operations. It is recommended to use the ``--copy-props`` parameter '
+        'instead to control copying of metadata properties. '
+        'If ``--metadata-directive`` is set, the ``--copy-props`` parameter '
+        'will be disabled and will have no affect on the transfer.'
+    )
+}
+
+
+COPY_PROPS = {
+    'name': 'copy-props',
+    'choices': ['none', 'metadata-directive', 'default'],
+    'default': 'default', 'help_text': (
+        'Determines which properties are copied from the source S3 object. '
+        'This parameter only applies for S3 to S3 copies. Valid values are: '
+        '<ul>'
+        '<li>``none`` - Do not copy any of the properties from the source '
+        'S3 object.</li>'
+        '<li>``metadata-directive`` - Copies the following properties from '
+        'the source S3 object: '
+        '``content-type``, ``content-language``, ``content-encoding``, '
+        '``content-disposition``, ``cache-control``, ``--expires``, and '
+        '``metadata``</li>'
+        '<li>``default`` - The default value. Copies tags and properties '
+        'covered under the ``metadata-directive`` value from the '
+        'source S3 object.</li>'
+        '</ul>'
+        'In order to copy the appropriate properties for multipart copies, '
+        'some of the options may require additional API calls if a multipart '
+        'copy is involved. Specifically:'
+        '<ul>'
+        '<li>``metadata-directive`` may require additional ``HeadObject`` '
+        'API calls.</li>'
+        '<li>``default`` may require additional ``HeadObject``, '
+        '``GetObjectTagging``, and ``PutObjectTagging`` API calls. Note this'
+        ' list of API calls may grow in the future in order to ensure '
+        'multipart copies preserve the exact properties a ``CopyObject`` '
+        'API call would preserve.</li>'
+        '</ul>'
+        'If you want to guarantee no additional API calls are made other than '
+        'than the ones needed to perform the actual copy, set this option to '
+        '``none``.'
+    )
 }
 
 
@@ -431,27 +460,6 @@ REQUEST_PAYER = {
     )
 }
 
-VALIDATE_SAME_S3_PATHS = {
-    'name': 'validate-same-s3-paths', 'action': 'store_true',
-    'help_text': (
-        'Resolves the source and destination S3 URIs to their '
-        'underlying buckets and verifies that the file or object '
-        'is not being moved onto itself. If you are using any type '
-        'of access point ARNs or access point aliases in your S3 URIs, '
-        'we strongly recommended using this parameter to help prevent '
-        'accidental deletions of the source file or object. This '
-        'parameter resolves the underlying buckets of S3 access point '
-        'ARNs and aliases, S3 on Outposts access point ARNs, and '
-        'Multi-Region Access Point ARNs. S3 on Outposts access point '
-        'aliases are not supported. Instead of using this parameter, '
-        'you can set the environment variable '
-        '``AWS_CLI_S3_MV_VALIDATE_SAME_S3_PATHS`` to ``true``. '
-        'NOTE: Path validation requires making additional API calls. '
-        'Future updates to this path-validation mechanism might change '
-        'which API calls are made.'
-    )
-}
-
 TRANSFER_ARGS = [DRYRUN, QUIET, INCLUDE, EXCLUDE, ACL,
                  FOLLOW_SYMLINKS, NO_FOLLOW_SYMLINKS, NO_GUESS_MIME_TYPE,
                  SSE, SSE_C, SSE_C_KEY, SSE_KMS_KEY_ID, SSE_C_COPY_SOURCE,
@@ -463,17 +471,14 @@ TRANSFER_ARGS = [DRYRUN, QUIET, INCLUDE, EXCLUDE, ACL,
                  REQUEST_PAYER]
 
 
-def get_client(session, region, endpoint_url, verify, config=None):
-    return session.create_client('s3', region_name=region,
-                                 endpoint_url=endpoint_url, verify=verify,
-                                 config=config)
-
-
 class S3Command(BasicCommand):
     def _run_main(self, parsed_args, parsed_globals):
-        self.client = get_client(self._session, parsed_globals.region,
-                                 parsed_globals.endpoint_url,
-                                 parsed_globals.verify_ssl)
+        params = {
+            'region': parsed_globals.region,
+            'endpoint_url': parsed_globals.endpoint_url,
+            'verify_ssl': parsed_globals.verify_ssl,
+        }
+        self.client = ClientFactory(self._session).create_client(params)
 
 
 class ListCommand(S3Command):
@@ -659,7 +664,7 @@ class WebsiteCommand(S3Command):
         # bucketname
         #
         # We also strip off the trailing slash if a user
-        # accidentally appends a slash.
+        # accidently appends a slash.
         if path.startswith('s3://'):
             path = path[5:]
         if path.endswith('/'):
@@ -673,8 +678,8 @@ class PresignCommand(S3Command):
     DESCRIPTION = (
         "Generate a pre-signed URL for an Amazon S3 object. This allows "
         "anyone who receives the pre-signed URL to retrieve the S3 object "
-        "with an HTTP GET request. For sigv4 requests the region needs to be "
-        "configured explicitly."
+        "with an HTTP GET request. All presigned URL's now use sigv4 "
+        "so the region needs to be configured explicitly."
     )
     USAGE = "<S3Uri>"
     ARG_TABLE = [{'name': 'path',
@@ -683,7 +688,7 @@ class PresignCommand(S3Command):
                   'cli_type_name': 'integer',
                   'help_text': (
                       'Number of seconds until the pre-signed '
-                      'URL expires.  Default is 3600 seconds.')}]
+                      'URL expires.  Default is 3600 seconds. Maximum is 604800 seconds.')}]
 
     def _run_main(self, parsed_args, parsed_globals):
         super(PresignCommand, self)._run_main(parsed_args, parsed_globals)
@@ -705,35 +710,21 @@ class S3TransferCommand(S3Command):
     def _run_main(self, parsed_args, parsed_globals):
         super(S3TransferCommand, self)._run_main(parsed_args, parsed_globals)
         self._convert_path_args(parsed_args)
-        params = self._build_call_parameters(parsed_args, {})
-        cmd_params = CommandParameters(self.NAME, params,
-                                       self.USAGE,
-                                       self._session,
-                                       parsed_globals)
-        cmd_params.add_region(parsed_globals)
-        cmd_params.add_endpoint_url(parsed_globals)
-        cmd_params.add_verify_ssl(parsed_globals)
-        cmd_params.add_page_size(parsed_args)
-        cmd_params.add_paths(parsed_args.paths)
-
-        runtime_config = transferconfig.RuntimeConfig().build_config(
-            **self._session.get_scoped_config().get('s3', {}))
-        cmd = CommandArchitecture(self._session, self.NAME,
-                                  cmd_params.parameters,
-                                  runtime_config)
-        cmd.set_clients()
+        params = self._get_params(parsed_args, parsed_globals)
+        source_client, transfer_client = self._get_source_and_transfer_clients(
+            params=params
+        )
+        transfer_manager = self._get_transfer_manager(
+            params=params,
+            botocore_transfer_client=transfer_client
+        )
+        cmd = CommandArchitecture(
+            self._session, self.NAME, params,
+            transfer_manager, source_client, transfer_client
+        )
         cmd.create_instructions()
         return cmd.run()
 
-    def _build_call_parameters(self, args, command_params):
-        """
-        This takes all of the commands in the name space and puts them
-        into a dictionary
-        """
-        for name, value in vars(args).items():
-            command_params[name] = value
-        return command_params
-
     def _convert_path_args(self, parsed_args):
         if not isinstance(parsed_args.paths, list):
             parsed_args.paths = [parsed_args.paths]
@@ -745,6 +736,36 @@ class S3TransferCommand(S3Command):
                 new_path = enc_path.decode('utf-8')
                 parsed_args.paths[i] = new_path
 
+    def _get_params(self, parsed_args, parsed_globals):
+        cmd_params = CommandParameters(
+            self.NAME, vars(parsed_args), self.USAGE)
+        cmd_params.add_region(parsed_globals)
+        cmd_params.add_endpoint_url(parsed_globals)
+        cmd_params.add_verify_ssl(parsed_globals)
+        cmd_params.add_sign_request(parsed_globals)
+        cmd_params.add_page_size(parsed_args)
+        cmd_params.add_paths(parsed_args.paths)
+        return cmd_params.parameters
+
+    def _get_source_and_transfer_clients(self, params):
+        client_factory = ClientFactory(self._session)
+        source_client = client_factory.create_client(
+            params, is_source_client=True)
+        transfer_client = client_factory.create_client(params)
+        return source_client, transfer_client
+
+    def _get_transfer_manager(self, params, botocore_transfer_client):
+        runtime_config = self._get_runtime_config()
+        return TransferManagerFactory(self._session).create_transfer_manager(
+            params=params,
+            runtime_config=runtime_config,
+            botocore_client=botocore_transfer_client,
+        )
+
+    def _get_runtime_config(self):
+        return transferconfig.RuntimeConfig().build_config(
+            **self._session.get_scoped_config().get('s3', {}))
+
 
 class CpCommand(S3TransferCommand):
     NAME = 'cp'
@@ -754,17 +775,19 @@ class CpCommand(S3TransferCommand):
             "or <S3Uri> <S3Uri>"
     ARG_TABLE = [{'name': 'paths', 'nargs': 2, 'positional_arg': True,
                   'synopsis': USAGE}] + TRANSFER_ARGS + \
-                [METADATA, METADATA_DIRECTIVE, EXPECTED_SIZE, RECURSIVE]
+                [METADATA, COPY_PROPS, METADATA_DIRECTIVE, EXPECTED_SIZE,
+                 RECURSIVE]
 
 
 class MvCommand(S3TransferCommand):
     NAME = 'mv'
-    DESCRIPTION = BasicCommand.FROM_FILE('s3', 'mv', '_description.rst')
+    DESCRIPTION = "Moves a local file or S3 object to " \
+                  "another location locally or in S3."
     USAGE = "<LocalPath> <S3Uri> or <S3Uri> <LocalPath> " \
             "or <S3Uri> <S3Uri>"
     ARG_TABLE = [{'name': 'paths', 'nargs': 2, 'positional_arg': True,
                   'synopsis': USAGE}] + TRANSFER_ARGS +\
-                [METADATA, METADATA_DIRECTIVE, RECURSIVE, VALIDATE_SAME_S3_PATHS]
+                [METADATA, COPY_PROPS, METADATA_DIRECTIVE, RECURSIVE]
 
 
 class RmCommand(S3TransferCommand):
@@ -786,7 +809,7 @@ class SyncCommand(S3TransferCommand):
             "<LocalPath> or <S3Uri> <S3Uri>"
     ARG_TABLE = [{'name': 'paths', 'nargs': 2, 'positional_arg': True,
                   'synopsis': USAGE}] + TRANSFER_ARGS + \
-                [METADATA, METADATA_DIRECTIVE]
+                [METADATA, COPY_PROPS, METADATA_DIRECTIVE]
 
 
 class MbCommand(S3Command):
@@ -799,7 +822,9 @@ class MbCommand(S3Command):
         super(MbCommand, self)._run_main(parsed_args, parsed_globals)
 
         if not parsed_args.path.startswith('s3://'):
-            raise TypeError("%s\nError: Invalid argument type" % self.USAGE)
+            raise ParamValidationError(
+                "%s\nError: Invalid argument type" % self.USAGE
+            )
         bucket, _ = split_s3_bucket_key(parsed_args.path)
 
         bucket_config = {'LocationConstraint': self.client.meta.region_name}
@@ -837,12 +862,16 @@ class RbCommand(S3Command):
         super(RbCommand, self)._run_main(parsed_args, parsed_globals)
 
         if not parsed_args.path.startswith('s3://'):
-            raise TypeError("%s\nError: Invalid argument type" % self.USAGE)
+            raise ParamValidationError(
+                "%s\nError: Invalid argument type" % self.USAGE
+            )
         bucket, key = split_s3_bucket_key(parsed_args.path)
 
         if key:
-            raise ValueError('Please specify a valid bucket name only.'
-                             ' E.g. s3://%s' % bucket)
+            raise ParamValidationError(
+                'Please specify a valid bucket name only. '
+                'E.g. s3://%s' % bucket
+            )
 
         if parsed_args.force:
             self._force(parsed_args.path, parsed_globals)
@@ -878,44 +907,15 @@ class CommandArchitecture(object):
     list of instructions to wire together an assortment of generators to
     perform the command.
     """
-    def __init__(self, session, cmd, parameters, runtime_config=None):
+    def __init__(self, session, cmd, parameters, transfer_manager,
+                 source_client, transfer_client):
         self.session = session
         self.cmd = cmd
         self.parameters = parameters
         self.instructions = []
-        self._runtime_config = runtime_config
-        self._endpoint = None
-        self._source_endpoint = None
-        self._client = None
-        self._source_client = None
-
-    def set_clients(self):
-        client_config = None
-        if self.parameters.get('sse') == 'aws:kms':
-            client_config = Config(signature_version='s3v4')
-        self._client = get_client(
-            self.session,
-            region=self.parameters['region'],
-            endpoint_url=self.parameters['endpoint_url'],
-            verify=self.parameters['verify_ssl'],
-            config=client_config
-        )
-        self._source_client = get_client(
-            self.session,
-            region=self.parameters['region'],
-            endpoint_url=self.parameters['endpoint_url'],
-            verify=self.parameters['verify_ssl'],
-            config=client_config
-        )
-        if self.parameters['source_region']:
-            if self.parameters['paths_type'] == 's3s3':
-                self._source_client = get_client(
-                    self.session,
-                    region=self.parameters['source_region'],
-                    endpoint_url=None,
-                    verify=self.parameters['verify_ssl'],
-                    config=client_config
-                )
+        self._transfer_manager = transfer_manager
+        self._source_client = source_client
+        self._client = transfer_client
 
     def create_instructions(self):
         """
@@ -1039,9 +1039,8 @@ class CommandArchitecture(object):
         file_info_builder = FileInfoBuilder(
             self._client, self._source_client, self.parameters)
 
-        s3_transfer_handler = S3TransferHandlerFactory(
-            self.parameters, self._runtime_config)(
-                self._client, result_queue)
+        s3_transfer_handler = S3TransferHandlerFactory(self.parameters)(
+            self._transfer_manager, result_queue)
 
         sync_strategies = self.choose_sync_strategies()
 
@@ -1142,13 +1141,22 @@ class CommandArchitecture(object):
             )
 
 
+# TODO: This class is fairly quirky in the sense that it is both a builder
+#  and a data object. In the future we should make the following refactorings
+#  to the class:
+#    1. Hoist all of the building logic into a builder/factory class that
+#    builds the parameters that will be passed around to all of the
+#    abstractions used by the S3 commands
+#    2. Make the CommandParameters class a dataclass that properly defines
+#    the various valid parameters. The difficult part right now is the
+#    parameters property right now is a dictionary that holds arbitrary
+#    key/value pairs making it difficult to know what is supported.
 class CommandParameters(object):
     """
     This class is used to do some initial error based on the
     parameters and arguments passed to the command line.
     """
-    def __init__(self, cmd, parameters, usage,
-                 session=None, parsed_globals=None):
+    def __init__(self, cmd, parameters, usage):
         """
         Stores command name and parameters.  Ensures that the ``dir_op`` flag
         is true if a certain command is being used.
@@ -1161,8 +1169,6 @@ class CommandParameters(object):
         self.cmd = cmd
         self.parameters = parameters
         self.usage = usage
-        self._session = session
-        self._parsed_globals = parsed_globals
         if 'dir_op' not in parameters:
             self.parameters['dir_op'] = False
         if 'follow_symlinks' not in parameters:
@@ -1200,7 +1206,7 @@ class CommandParameters(object):
         if self.cmd == 'sync' and \
             (self._is_s3express_path(self.parameters['src']) or
              self._is_s3express_path(self.parameters['dest'])):
-            raise ValueError(
+            raise ParamValidationError(
                 "Cannot use sync command with a directory bucket."
             )
 
@@ -1214,7 +1220,7 @@ class CommandParameters(object):
         self.parameters['is_stream'] = False
         if self.parameters['src'] == '-' or self.parameters['dest'] == '-':
             if self.cmd != 'cp' or self.parameters.get('dir_op'):
-                raise ValueError(
+                raise ParamValidationError(
                     "Streaming currently is only compatible with "
                     "non-recursive cp commands"
                 )
@@ -1225,12 +1231,10 @@ class CommandParameters(object):
     def _validate_path_args(self):
         # If we're using a mv command, you can't copy the object onto itself.
         params = self.parameters
-        if self.cmd == 'mv' and params['paths_type']=='s3s3':
-            self._raise_if_mv_same_paths(params['src'], params['dest'])
-            if self._should_validate_same_underlying_s3_paths():
-                self._validate_same_underlying_s3_paths()
-            if self._should_emit_validate_s3_paths_warning():
-                self._emit_validate_s3_paths_warning()
+        if self.cmd == 'mv' and self._same_path(params['src'], params['dest']):
+            raise ParamValidationError(
+                "Cannot mv a file onto itself: '%s' - '%s'" % (
+                params['src'], params['dest']))
 
         # If the user provided local path does not exist, hard fail because
         # we know that we will not be able to upload the file.
@@ -1255,66 +1259,6 @@ class CommandParameters(object):
             src_base = os.path.basename(src)
             return src == os.path.join(dest, src_base)
 
-    def _same_key(self, src, dest):
-        _, src_key = split_s3_bucket_key(src)
-        _, dest_key = split_s3_bucket_key(dest)
-        return self._same_path(f'/{src_key}', f'/{dest_key}')
-
-    def _validate_same_s3_paths_enabled(self):
-        validate_env_var = ensure_boolean(
-            os.environ.get('AWS_CLI_S3_MV_VALIDATE_SAME_S3_PATHS'))
-        return (self.parameters.get('validate_same_s3_paths') or
-                validate_env_var)
-
-    def _should_emit_validate_s3_paths_warning(self):
-        is_same_key = self._same_key(
-            self.parameters['src'], self.parameters['dest'])
-        src_has_underlying_path = S3PathResolver.has_underlying_s3_path(
-            self.parameters['src'])
-        dest_has_underlying_path = S3PathResolver.has_underlying_s3_path(
-            self.parameters['dest'])
-        return (is_same_key and not self._validate_same_s3_paths_enabled() and
-                (src_has_underlying_path or dest_has_underlying_path))
-
-    def _emit_validate_s3_paths_warning(self):
-        msg = (
-            "warning: Provided s3 paths may resolve to same underlying "
-            "s3 object(s) and result in deletion instead of being moved. "
-            "To resolve and validate underlying s3 paths are not the same, "
-            "specify the --validate-same-s3-paths flag or set the "
-            "AWS_CLI_S3_MV_VALIDATE_SAME_S3_PATHS environment variable to true. "
-            "To resolve s3 outposts access point path, the arn must be "
-            "used instead of the alias.\n"
-        )
-        uni_print(msg, sys.stderr)
-
-    def _should_validate_same_underlying_s3_paths(self):
-        is_same_key = self._same_key(
-            self.parameters['src'], self.parameters['dest'])
-        return is_same_key and self._validate_same_s3_paths_enabled()
-
-    def _validate_same_underlying_s3_paths(self):
-        src_paths = S3PathResolver.from_session(
-            self._session,
-            self.parameters.get('source_region', self._parsed_globals.region),
-            self._parsed_globals.verify_ssl
-        ).resolve_underlying_s3_paths(self.parameters['src'])
-        dest_paths = S3PathResolver.from_session(
-            self._session,
-            self._parsed_globals.region,
-            self._parsed_globals.verify_ssl
-        ).resolve_underlying_s3_paths(self.parameters['dest'])
-        for src_path in src_paths:
-            for dest_path in dest_paths:
-                self._raise_if_mv_same_paths(src_path, dest_path)
-
-    def _raise_if_mv_same_paths(self, src, dest):
-        if self._same_path(src, dest):
-            raise ValueError(
-                "Cannot mv a file onto itself: "
-                f"{self.parameters['src']} - {self.parameters['dest']}"
-            )
-
     def _normalize_s3_trailing_slash(self, paths):
         for i, path in enumerate(paths):
             if path.startswith('s3://'):
@@ -1347,7 +1291,9 @@ class CommandParameters(object):
         if self.cmd in template_type[paths_type]:
             self.parameters['paths_type'] = paths_type
         else:
-            raise TypeError("%s\nError: Invalid argument type" % usage)
+            raise ParamValidationError(
+                "%s\nError: Invalid argument type" % usage
+            )
 
     def add_region(self, parsed_globals):
         self.parameters['region'] = parsed_globals.region
@@ -1365,6 +1311,10 @@ class CommandParameters(object):
     def add_verify_ssl(self, parsed_globals):
         self.parameters['verify_ssl'] = parsed_globals.verify_ssl
 
+    def add_sign_request(self, parsed_globals):
+        self.parameters['sign_request'] = getattr(
+            parsed_globals, 'sign_request', True)
+
     def add_page_size(self, parsed_args):
         self.parameters['page_size'] = getattr(parsed_args, 'page_size', None)
 
@@ -1379,21 +1329,21 @@ class CommandParameters(object):
         sse_c_key_type_param = '--' + sse_c_key_type.replace('_', '-')
         if self.parameters.get(sse_c_type):
             if not self.parameters.get(sse_c_key_type):
-                raise ValueError(
-                    'It %s is specified, %s must be specified '
+                raise ParamValidationError(
+                    'If %s is specified, %s must be specified '
                     'as well.' % (sse_c_type_param, sse_c_key_type_param)
                 )
         if self.parameters.get(sse_c_key_type):
             if not self.parameters.get(sse_c_type):
-                raise ValueError(
-                    'It %s is specified, %s must be specified '
+                raise ParamValidationError(
+                    'If %s is specified, %s must be specified '
                     'as well.' % (sse_c_key_type_param, sse_c_type_param)
                 )
 
     def _validate_sse_c_copy_source_for_paths(self):
         if self.parameters.get('sse_c_copy_source'):
             if self.parameters['paths_type'] != 's3s3':
-                raise ValueError(
+                raise ParamValidationError(
                     '--sse-c-copy-source is only supported for '
                     'copy operations.'
                 )
