@@ -17,15 +17,14 @@ import mimetypes
 import errno
 import os
 import re
-import time
 from collections import namedtuple, deque
 
 from dateutil.parser import parse
 from dateutil.tz import tzlocal, tzutc
-from s3transfer.subscribers import BaseSubscriber
 
 from awscli.compat import bytes_print
 from awscli.compat import queue
+from awscli.customizations.exceptions import ParamValidationError
 
 LOGGER = logging.getLogger(__name__)
 HUMANIZE_SUFFIXES = ('KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB')
@@ -44,7 +43,7 @@ SIZE_SUFFIX = {
     'tib': 1024 ** 4,
 }
 _S3_ACCESSPOINT_TO_BUCKET_KEY_REGEX = re.compile(
-    r'^(?P<bucket>arn:(aws).*:s3:[a-z\-0-9]+:[0-9]{12}:accesspoint[:/][^/]+)/?'
+    r'^(?P<bucket>arn:(aws).*:s3:[a-z\-0-9]*:[0-9]{12}:accesspoint[:/][^/]+)/?'
     r'(?P<key>.*)$'
 )
 _S3_OUTPOST_TO_BUCKET_KEY_REGEX = re.compile(
@@ -96,13 +95,13 @@ def human_readable_size(value):
             return '%.1f %s' % ((base * bytes_int / unit), suffix)
 
 
-def human_readable_to_bytes(value):
-    """Converts a human readable size to bytes.
+def human_readable_to_int(value):
+    """Converts a human readable size to int.
 
     :param value: A string such as "10MB".  If a suffix is not included,
         then the value is assumed to be an integer representing the size
         in bytes.
-    :returns: The converted value in bytes as an integer
+    :returns: The converted value as an integer.
 
     """
     value = value.lower()
@@ -134,7 +133,7 @@ class AppendFilter(argparse.Action):
     --exclude and the value will be the rule to apply.  This will
     format all of the rules inputted into the command line
     in a way compatible with the Filter class.  Note that rules that
-    appear later in the command line take preferance over rulers that
+    appear later in the command line take preference over rulers that
     appear earlier.
     """
     def __call__(self, parser, namespace, values, option_string=None):
@@ -198,17 +197,14 @@ def block_unsupported_resources(s3_path):
     # AWS CLI s3 commands don't support object lambdas only direct API calls
     # are available for such resources
     if _S3_OBJECT_LAMBDA_TO_BUCKET_KEY_REGEX.match(s3_path):
-        # In AWS CLI v2 we should use
-        # awscli.customizations.exceptions.ParamValidationError
-        # instead of ValueError
-        raise ValueError(
+        raise ParamValidationError(
             's3 commands do not support S3 Object Lambda resources. '
             'Use s3api commands instead.'
         )
     # AWS S3 API and AWS CLI s3 commands don't support Outpost bucket ARNs
     # only s3control API supports them so far
     if _S3_OUTPOST_BUCKET_ARN_TO_BUCKET_KEY_REGEX.match(s3_path):
-        raise ValueError(
+        raise ParamValidationError(
             's3 commands do not support Outpost Bucket ARNs. '
             'Use s3control commands instead.'
         )
@@ -338,9 +334,9 @@ def guess_content_type(filename):
     """
     try:
         return mimetypes.guess_type(filename)[0]
-    # This catches a bug in the mimetype libary where some MIME types
+    # This catches a bug in the mimetype library where some MIME types
     # specifically on windows machines cause a UnicodeDecodeError
-    # because the MIME type in the Windows registery has an encoding
+    # because the MIME type in the Windows registry has an encoding
     # that cannot be properly encoded using the default system encoding.
     # https://bugs.python.org/issue9291
     #
@@ -481,6 +477,16 @@ class RequestParamsMapper(object):
         cls._set_sse_c_request_params(request_params, cli_params)
         cls._set_request_payer_param(request_params, cli_params)
 
+    @classmethod
+    def map_get_object_tagging_params(cls, request_params, cli_params):
+        """Map CLI params to GetObjectTagging request params"""
+        cls._set_request_payer_param(request_params, cli_params)
+
+    @classmethod
+    def map_put_object_tagging_params(cls, request_params, cli_params):
+        """Map CLI params to PutObjectTagging request params"""
+        cls._set_request_payer_param(request_params, cli_params)
+
     @classmethod
     def map_copy_object_params(cls, request_params, cli_params):
         """Map CLI params to CopyObject request params"""
@@ -563,8 +569,9 @@ class RequestParamsMapper(object):
                 try:
                     permission, grantee = grant.split('=', 1)
                 except ValueError:
-                    raise ValueError('grants should be of the form '
-                                     'permission=principal')
+                    raise ParamValidationError(
+                        'grants should be of the form permission=principal'
+                    )
                 request_params[cls._permission_to_param(permission)] = grantee
 
     @classmethod
@@ -577,8 +584,9 @@ class RequestParamsMapper(object):
             return 'GrantReadACP'
         if permission == 'writeacl':
             return 'GrantWriteACP'
-        raise ValueError('permission must be one of: '
-                         'read|readacl|writeacl|full')
+        raise ParamValidationError(
+            'permission must be one of: read|readacl|writeacl|full'
+        )
 
     @classmethod
     def _set_metadata_params(cls, request_params, cli_params):
@@ -625,151 +633,6 @@ class RequestParamsMapper(object):
         cls._set_sse_c_copy_source_request_params(request_params, cli_params)
 
 
-class ProvideSizeSubscriber(BaseSubscriber):
-    """
-    A subscriber which provides the transfer size before it's queued.
-    """
-    def __init__(self, size):
-        self.size = size
-
-    def on_queued(self, future, **kwargs):
-        future.meta.provide_transfer_size(self.size)
-
-
-# TODO: Eventually port this down to the BaseSubscriber or a new subscriber
-# class in s3transfer. The functionality is very convenient but may need
-# some further design decisions to make it a feature in s3transfer.
-class OnDoneFilteredSubscriber(BaseSubscriber):
-    """Subscriber that differentiates between successes and failures
-
-    It is really a convenience class so developers do not have to have
-    to constantly remember to have a general try/except around future.result()
-    """
-    def on_done(self, future, **kwargs):
-        future_exception = None
-        try:
-
-            future.result()
-        except Exception as e:
-            future_exception = e
-        # If the result propogates an error, call the on_failure
-        # method instead.
-        if future_exception:
-            self._on_failure(future, future_exception)
-        else:
-            self._on_success(future)
-
-    def _on_success(self, future):
-        pass
-
-    def _on_failure(self, future, e):
-        pass
-
-
-class DeleteSourceSubscriber(OnDoneFilteredSubscriber):
-    """A subscriber which deletes the source of the transfer."""
-    def _on_success(self, future):
-        try:
-            self._delete_source(future)
-        except Exception as e:
-            future.set_exception(e)
-
-    def _delete_source(self, future):
-        raise NotImplementedError('_delete_source()')
-
-
-class DeleteSourceObjectSubscriber(DeleteSourceSubscriber):
-    """A subscriber which deletes an object."""
-    def __init__(self, client):
-        self._client = client
-
-    def _get_bucket(self, call_args):
-        return call_args.bucket
-
-    def _get_key(self, call_args):
-        return call_args.key
-
-    def _delete_source(self, future):
-        call_args = future.meta.call_args
-        delete_object_kwargs = {
-            'Bucket': self._get_bucket(call_args),
-            'Key': self._get_key(call_args)
-        }
-        if call_args.extra_args.get('RequestPayer'):
-            delete_object_kwargs['RequestPayer'] = call_args.extra_args[
-                'RequestPayer']
-        self._client.delete_object(**delete_object_kwargs)
-
-
-class DeleteCopySourceObjectSubscriber(DeleteSourceObjectSubscriber):
-    """A subscriber which deletes the copy source."""
-    def _get_bucket(self, call_args):
-        return call_args.copy_source['Bucket']
-
-    def _get_key(self, call_args):
-        return call_args.copy_source['Key']
-
-
-class DeleteSourceFileSubscriber(DeleteSourceSubscriber):
-    """A subscriber which deletes a file."""
-    def _delete_source(self, future):
-        os.remove(future.meta.call_args.fileobj)
-
-
-class BaseProvideContentTypeSubscriber(BaseSubscriber):
-    """A subscriber that provides content type when creating s3 objects"""
-
-    def on_queued(self, future, **kwargs):
-        guessed_type = guess_content_type(self._get_filename(future))
-        if guessed_type is not None:
-            future.meta.call_args.extra_args['ContentType'] = guessed_type
-
-    def _get_filename(self, future):
-        raise NotImplementedError('_get_filename()')
-
-
-class ProvideUploadContentTypeSubscriber(BaseProvideContentTypeSubscriber):
-    def _get_filename(self, future):
-        return future.meta.call_args.fileobj
-
-
-class ProvideCopyContentTypeSubscriber(BaseProvideContentTypeSubscriber):
-    def _get_filename(self, future):
-        return future.meta.call_args.copy_source['Key']
-
-
-class ProvideLastModifiedTimeSubscriber(OnDoneFilteredSubscriber):
-    """Sets utime for a downloaded file"""
-    def __init__(self, last_modified_time, result_queue):
-        self._last_modified_time = last_modified_time
-        self._result_queue = result_queue
-
-    def _on_success(self, future, **kwargs):
-        filename = future.meta.call_args.fileobj
-        try:
-            last_update_tuple = self._last_modified_time.timetuple()
-            mod_timestamp = time.mktime(last_update_tuple)
-            set_file_utime(filename, int(mod_timestamp))
-        except Exception as e:
-            warning_message = (
-                'Successfully Downloaded %s but was unable to update the '
-                'last modified time. %s' % (filename, e))
-            self._result_queue.put(create_warning(filename, warning_message))
-
-
-class DirectoryCreatorSubscriber(BaseSubscriber):
-    """Creates a directory to download if it does not exist"""
-    def on_queued(self, future, **kwargs):
-        d = os.path.dirname(future.meta.call_args.fileobj)
-        try:
-            if not os.path.exists(d):
-                os.makedirs(d)
-        except OSError as e:
-            if not e.errno == errno.EEXIST:
-                raise CreateDirectoryError(
-                    "Could not create directory %s: %s" % (d, e))
-
-
 class NonSeekableStream(object):
     """Wrap a file like object as a non seekable stream.
 
